SAUDC0 = $15
SAUDF0 = $17
SAUDV0 = $19


;--------------------------------------------------------------------------
; songPlayer
;--------------------------------------------------------------------------
; Plays up to two pre-programmed patterns simlutaneously.
;
; Call this once per screen-draw.
;
; Keep this at the beginning of songplay.h to avoid an extra JMP
;--------------------------------------------------------------------------
songPlayer

; Now the actual player
;--------------------------------------------------------------------------

	ldy measure	
	ldx song1,y

	; Check to see if the end of the song was reached
	cpx #255
	bne notEndOfSong
	
	; Go back to the first measure
	ldy #0
	sty measure
	ldx song1,y

notEndOfSong

	lda #0 ; oscillator zero
	jsr playPattern

	ldy measure	
	ldx song2,y

	lda #1 ; oscillator one
	jsr playPattern

	jmp reentry		; Jump to the point where the Berzerk
					; code is re-entered
;--------------------------------------------------




tempo


; Tempo
;--------------------------------------------------------------------------
	; I'm using the top 3 bits of the beat as a tempo counter.
	; I increase the beat number every 3 screen draws.

	lda beat
	and #%11100000
	beq tempoStep1

	cmp #%00100000
	beq tempoStep2

	;reset tempo count
	lda beat
	and #%00011111
	sta beat

	tax
	inx
	stx beat
	cpx #32
	bne quitTempo

	inc measure

	lda #0
	sta beat

	jmp quitTempo

tempoStep1
	lda beat
	ora #%00100000
	sta beat
	jmp quitTempo

tempoStep2
	lda beat
	ora #%01100000
	sta beat
	
quitTempo

	sta WSYNC
	sta WSYNC

	jmp reentry2






soundTypeArray
	byte 4,6,7,8
	byte 15,12,1,3







;--------------------------------------------------------------------------
; Accent Reader
;--------------------------------------------------------------------------
; Each set of pattern data is followed by 4 accept bytes.
; Each bit in order represents the accent (on or off)
; of its corresponding 32nd note.  This function
; returns the attenuation of a note in a pattern.
;
; - temp16 must contain an indirect pointer to the pattern data
; - Y must contain the current 32nd note (beat)
;
; = will return the volume in ACC
;
; changes X,Y,ACC
;--------------------------------------------------------------------------
bitMaskArray
	byte #%10000000
	byte #%01000000
	byte #%00100000
	byte #%00010000
	byte #%00001000
	byte #%00000100
	byte #%00000010
	byte #%00000001

accentReader
	; There are 4 accent bytes.  Get the
	; correct one.
	tya

	; Now set up a mask for the bit
	and #%00000111
	tax
	tya

	and #%00011000
	lsr
	lsr
	lsr

	; The 4 accent bytes are after the 32 bytes
	; that make up the pattern.
	clc
	adc #32
	tay
	
	lda (temp16L),y
	and bitMaskArray,x
	beq noAccent

	; It's an Accent, so don't attenuate
	lda #15

	jmp rtnAccentReader

noAccent

	; No accent, so use a lower volume
	lda #5

	;rts ; accentReader
	jmp rtnAccentReader


;--------------------------------------------------------------------------
; Extract Pattern Data
;--------------------------------------------------------------------------
; Each byte of pattern data contains the frequency and 
; sound type data.  This function separates and decodes them.
;
; The encoding is: the 3 high bits contain the encoded sound
; type and the lower 5 bits contain the freq data.
;
; - ACC must contain pattern byte
;
; = ACC will return the freq
; = X will return the sound type
; 
; changes ACC,X,Y
;--------------------------------------------------------------------------
extractPatternData


	cmp #255
	beq noNote
	
	tax

	; Extract freq data and push it
	and #%00011111

	;jmp rtnExtractPatternData

	tay

	txa
	lsr
	lsr
	lsr
	lsr
	lsr
	tax
	
	lda soundTypeArray,x

	tax
	tya

	;rts
	jmp rtnExtractPatternData

noNote
	lda #255

	;rts ; extractPatternData : other rts
	jmp rtnExtractPatternData





;--------------------------------------------------------------------------
; playPattern
;--------------------------------------------------------------------------
; Plays a pattern
;
; - X should contain the offset in the patternArray of the pattern to play
; - ACC should contain the oscillator to be used (0 or 1)
;
;--------------------------------------------------------------------------
playPattern

	; save osc number
	sta temp

	; Get address of selected pattern
	lda patternArray,x
	sta temp16L
	inx
	lda patternArray,x
	sta temp16H

	; The variable, beat, contains the 32nd note
	; that the beat is currently on.
	lda beat
	and #%00011111
	tay

	lda (temp16L),y

	jmp extractPatternData
rtnExtractPatternData


	; Get the osc number again
	ldy temp

	sta SAUDF0,y
	stx SAUDC0,y	

	cmp #255
	beq noteOff1

	tax
	lda beat
	and #%00011111
	tay
	txa
	

	jmp accentReader
rtnAccentReader

	;rts

	; Get the osc number again
	ldy temp
	sta SAUDV0,y

	rts
noteOff1

	lda #0
	ldy temp
	sta SAUDV0,y

	rts ; playPattern




