; Zero page addresses for our RAM variables
;-------------------------------------------
frame equ $80
scanSec equ $81 ;1
count equ $82
bkColor equ $83



				
Start	   	
;-----------
;Clear memory, locate character graphics positions and such,
;initialize key memory values, start GameLoop.
;-------------------------------------------------------------

;the usual mumbo jumbo
;---------------------
	sei  	
	cld  		
	ldx #$FF
	txs  		
	lda #0
clear   
	sta 0,x
	dex
	bne clear	


;use cycle counting to get the GRP registers configured and located
;in exactly the right spot
;--------------------------------------
	sta WSYNC	;newline
	lda $54		;PRS a little delay
	lda #54		;triple copy sprites, spaced wide
	sta NUSIZ0	;for both player graphic registers
	sta NUSIZ1	
	lda #$31	;draw leftmost and rightmost 2 playfield bits: reflect
	sta PF0		;this turns left/right edges black
	sta CTRLPF	;this makes playfield reflective
	nop		;wait 2 cycles, we need them get our RESP0/1 right
	lda $54		;PRS a little delay
	lda $54		;PRS a little delay
	sta RESP0	;mark our left margin here in P0
	lda #$D0	;setup to shift over 3 pixels to the right
	sta RESP1	;set P1
	sta HMP0	;prep P0 for his HMOVE
	lda #$C0	;setup to shift P1 4 pixels to the right
	sta HMP1	;prep P1 for his HMOVE
	sta WSYNC	;newline
	sta HMOVE	;anchor them down




;----
;Initialize variables
;---------------------------------------------------------------------------

	; Set text color to 
	lda #%10010000
	sta COLUP0
	sta COLUP1

	ldx #0		;will load from left to right
	lda #0

	sta count

;-----------------
;Everything is set up, so loop through the basic game routines endlessly, 
;cutting out only when RESET is pressed.  One pass of this loop represents
;one display frame.
;-------------------------------------------------------------------------
GameLoop
	;jmp startGame	; DEBUG


	jsr VSync 	;start vertical retrace

	inc count

	lda count
	cmp #64
	beq section2

	lsr
	lsr
	and #%00001111
	ora #%10010000
	sta bkColor
	
	inc frame
	jsr Picture	;draw one screen
	jmp GameLoop    ;back to top


section2
	lda #0
	sta count

loop2

	inc count

	lda count
	cmp #64
	beq loop3
	lsr
	lsr
	and #%00001111
	ora #%10010000
	sta COLUP0
	sta COLUP1

	inc frame
	jsr Picture	;draw one screen
	jsr VSync 	;start vertical retrace
	jmp loop2    ;back to top

section3
	lda #64
	sta count

loop3

	dec count
	beq section4

	inc frame
	jsr Picture	;draw one screen
	jsr VSync 	;start vertical retrace
	jmp loop3    ;back to top

section4
	
	lda #64
	sta count

loop4
	dec count

	lda count
	beq section5
	lsr
	lsr
	and #%00001111
	ora #%10010000
	sta COLUP0
	sta COLUP1
	sta bkColor

	inc frame
	jsr Picture	;draw one screen
	jsr VSync 	;start vertical retrace
	jmp loop4    ;back to top

section5
	lda #64
	sta count

loop5

	lda #0
	sta COLUBK
	sta COLUPF
	sta COLUP0
	sta COLUP1
	sta bkColor

	dec count
	beq startGame

	inc frame
	jsr Picture	;draw one screen
	jsr VSync 	;start vertical retrace
	jmp loop5    ;back to top



startGame
	jmp myInit

VSync
;-------------------
	lda #2		;bit 1 needs to be 1 to start retrace
	sta VSYNC	;start retrace
	sta WSYNC 	;wait a few lines
	sta WSYNC 
	lda #44		;prepare timer to exit blank period
	sta TIM64T	;turn it on
	sta WSYNC 	;wait one more
	sta VSYNC 	;done with retrace, write 0 to bit 1
	rts 


	align 256
Picture
;----------------
;Here is where we step down the screen drawing everything
;--------------------------------------------------------

	lda #42  ;setup counter for rows of text
	sta scanSec	;store in zero page memory variable

Wait
	lda INTIM	;check timer for end of VBLANK period
	bne Wait	;loop until it reaches 0

	sta WSYNC	;newline
	lda #$80
	sta VBLANK  	;end screen blank

	lda #0
	sta GRP0
	sta GRP1
	sta COLUPF
	lda #255
	sta PF0
	sta PF1
	sta PF2




	ldx #0
	lda frame
	and #$01
	beq evenFrame
	jmp oddFrame


evenFrame	
;---------------------

	lda #$70 ;0111 1001
	sta HMP0
	sta HMP1
	sta WSYNC
	sta HMOVE
	
	sta WSYNC
	lda #$10
	sta HMP0
	sta HMP1
	sta WSYNC
	sta HMOVE

	clc

	jsr firstLines

evenLoop
	sta WSYNC

	lda graphics1,x
	sta GRP0
	lda graphics1+1,x
	sta GRP1

	ldy graphics1+3,x
	ldy graphics1+3,x
	ldy graphics1+3,x
	nop
	lda $54

	lda graphics1+2,x
	sta GRP0
	sty GRP1

	lda graphics1+4,x
	sta GRP0
	lda graphics1+5,x
	sta GRP1

	txa
	adc #6
	tax
	
	dec scanSec	;next row to print
	bne evenLoop	;loop until all 10 rows of text have been displayed

	lda #42
	sta scanSec
	ldx #0

evenLoop2
	sta WSYNC

	lda graphics3,x
	sta GRP0
	lda graphics3+1,x
	sta GRP1

	ldy graphics3+3,x
	ldy graphics3+3,x
	ldy graphics3+3,x
	nop
	lda $54

	lda graphics3+2,x
	sta GRP0
	sty GRP1

	lda graphics3+4,x
	sta GRP0
	lda graphics3+5,x
	sta GRP1

	txa
	adc #6
	tax
	
	dec scanSec	;next row to print
	bne evenLoop2	;loop until all 10 rows of text have been displayed

	sta WSYNC

	jmp overscan


oddFrame	
;---------------------

	lda #$90
	sta HMP0
	sta HMP1
	sta WSYNC
	sta HMOVE

	sta WSYNC
	lda #$F0
	sta HMP0
	sta HMP1
	sta WSYNC
	sta HMOVE

	clc

	jsr firstLines

oddLoop
	sta WSYNC

	lda graphics2+0,x
	sta GRP0
	lda graphics2+1,x
	sta GRP1

	ldy graphics2+3,x
	ldy graphics2+3,x
	ldy graphics2+3,x
	nop
	nop
	lda $54

	lda graphics2+2,x
	sta GRP0
	sty GRP1

	lda graphics2+4,x
	sta GRP0
	lda graphics2+5,x
	sta GRP1

	txa
	adc #6
	tax
	
	dec scanSec	;next row to print
	bne oddLoop	;loop until all 10 rows of text have been displayed

	lda #42
	sta scanSec
	ldx #0

oddLoop2
	sta WSYNC

	lda graphics4+0,x
	sta GRP0
	lda graphics4+1,x
	sta GRP1

	ldy graphics4+3,x
	ldy graphics4+3,x
	ldy graphics4+3,x
	nop
	nop
	lda $54

	lda graphics4+2,x
	sta GRP0
	sty GRP1

	lda graphics4+4,x
	sta GRP0
	lda graphics4+5,x
	sta GRP1

	txa
	adc #6
	tax
	
	dec scanSec	;next row to print
	bne oddLoop2	;loop until all 10 rows of text have been displayed



	jmp overscan





; ************** OVERSCAN ******************

; *** Handle the rest of the Scan Lines ***

overscan
	ldx #0
finish2
	sta WSYNC
	txa
	lsr
	lsr
	and #%00001111
	ora #%10010000
	and bkColor
	sta COLUPF
	inx
	cpx #53
	bne finish2


	lda #$82
	sta VBLANK	;finished a screen, blank the beam



	sta WSYNC	
	lda #33		; Use the timer to make sure overscan takes (34)
	sta TIM64T	; 30 scan lines.  29 scan lines * 76 = 2204 / 64 = 34.4

endOS	
	lda INTIM	; We finished, but wait for timer
	bne endOS	; by looping till zero
	
	sta WSYNC	; End last scanline

	lda #$82
	sta VBLANK
	lda #$02
	sta VBLANK

	rts		;done

;******************************************
firstLines
	ldx #52
finishEven
	sta WSYNC
	txa
	lsr
	lsr
	and #%00001111
	ora #%10010000
	and bkColor
	sta COLUPF
	dex
	bne finishEven
	rts

